#include <perl.h>
#include <perlio.h>
#include <perliol.h>

/*
 * this file defines a perl 'iolayer' named 'idamsg',
 * which outputs everything in the ida message log.
 * and adds this layer to the stdout, and stderr chain.
 */

#ifdef DYNAMIC_PERL
#include "redefperlasdll.h"
#include "perldllprocs.h"
#endif

#include "iolayer.h"

// ida includes
#include "kernwin.hpp"  // for msg()

SSize_t PerlIOidamsg_write(pTHX_ PerlIO *f, const void *vbuf, Size_t count)
{
    SSize_t total= 0;
    Size_t ofs= 0;
    while (ofs<count) {
        Size_t printed= msg("%.*s", count-ofs, (const char*)vbuf+ofs);
        ofs += printed;
        if (printed==0)
            break;
    }
    return total;
}
static PerlIO_funcs PerlIO_idamsg = {
    sizeof(PerlIO_funcs),        
    "idamsg",                    
    sizeof(_PerlIO),
    PERLIO_K_MULTIARG | PERLIO_K_RAW,
    PerlIOBase_pushed,         
    NULL,                       /* PerlIOBase_popped */
    NULL,                       /* PerlIOidamsg_open */
    NULL,                       /* PerlIOBase_binmode */
    NULL,                       /* no getarg needed */
    NULL,                       /* PerlIOidamsg_fileno */
    NULL,                       /* PerlIOidamsg_dup */
    NULL,                       /* PerlIOidamsg_read */
    NULL,                       /* PerlIOBase_unread */
    PerlIOidamsg_write,         /* PerlIOidamsg_write */
    NULL,                       /* PerlIOidamsg_seek */
    NULL,                       /* PerlIOidamsg_tell */
    NULL,                       /* PerlIOidamsg_close */
    NULL,                       /* PerlIOidamsg_flush */
    NULL,                       /* PerlIOidamsg_noop_fail - fill */
    NULL,                       /* PerlIOidamsg_eof */
    NULL,                       /* PerlIOBase_error */
    NULL,                       /* PerlIOBase_clearerr */
    NULL,                       /* PerlIOBase_setlinebuf */
    NULL,                       /* get_base */
    NULL,                       /* get_bufsiz */
    NULL,                       /* get_ptr */
    NULL,                       /* get_cnt */
    NULL,                       /* set_ptrcnt */
};

void init_idamsg_io(pTHX)
{
    PerlIO_define_layer(aTHX_ &PerlIO_idamsg);

//binmode ":idamsg", stdout;
    PerlIO_push(aTHX_ PerlIO_stdout(), &PerlIO_idamsg, "a", NULL);
//binmode ":idamsg", stderr;
    PerlIO_push(aTHX_ PerlIO_stderr(), &PerlIO_idamsg, "a", NULL);
}

